@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 /* FIXME: Should probably buffer calls to libtiff with TIFFSetWarningHandler
@@ -37,12 +37,12 @@
 
 struct _TiffDocumentClass
 {
-  GObjectClass parent_class;
+  EvDocumentClass parent_class;
 };
 
 struct _TiffDocument
 {
-  GObject parent_instance;
+  EvDocument parent_instance;
 
   TIFF *tiff;
   gint n_pages;
@@ -53,7 +53,6 @@ struct _TiffDocument
 
 typedef struct _TiffDocumentClass TiffDocumentClass;
 
-static void tiff_document_document_iface_init (EvDocumentIface *iface);
 static void tiff_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface);
 static void tiff_document_document_file_exporter_iface_init (EvFileExporterIface *iface);
 
@@ -260,10 +259,10 @@ tiff_document_render (EvDocument      *document,
 	rowstride = cairo_format_stride_for_width (CAIRO_FORMAT_RGB24, width);
 #else
 	rowstride = width * 4;
-	if (rowstride / 4 != width)
-		/* overflow */
-		return NULL;                
 #endif
+	if (rowstride / 4 != width)
+		/* overflow, or cairo was changed in an unsupported way */
+		return NULL;                
 	
 	bytes = height * rowstride;
 	if (bytes / rowstride != height)
@@ -292,16 +291,15 @@ tiff_document_render (EvDocument      *document,
 	*/
 	p = pixels;
 	while (p < pixels + bytes) {
-		uint32 pixel = *(uint32 *)p;
-		int r = TIFFGetR(pixel);
-		int g = TIFFGetG(pixel);
-		int b = TIFFGetB(pixel);
-		int a = TIFFGetA(pixel);
-		
-		*p++ = b;
-		*p++ = g;
-		*p++ = r;
-		*p++ = a;
+		guint32 *pixel = (guint32*)p;
+		guint8 r = TIFFGetR(*pixel);
+		guint8 g = TIFFGetG(*pixel);
+		guint8 b = TIFFGetB(*pixel);
+		guint8 a = TIFFGetA(*pixel);
+
+		*pixel = (a << 24) | (r << 16) | (g << 8) | b;
+
+		p += 4;
 	}
 
 	rotated_surface = ev_document_misc_surface_rotate_and_scale (surface,
@@ -385,6 +383,21 @@ tiff_document_render_pixbuf (EvDocument      *document,
 	return rotated_pixbuf;
 }
 
+static gchar *
+tiff_document_get_page_label (EvDocument *document,
+			      EvPage     *page)
+{
+	TiffDocument *tiff_document = TIFF_DOCUMENT (document);
+	static gchar *label;
+
+	if (TIFFGetField (tiff_document->tiff, TIFFTAG_PAGENAME, &label) &&
+	    g_utf8_validate (label, -1, NULL)) {
+		return g_strdup (label);
+	}
+
+	return NULL;
+}
+
 static void
 tiff_document_finalize (GObject *object)
 {
@@ -401,47 +414,17 @@ tiff_document_finalize (GObject *object)
 static void
 tiff_document_class_init (TiffDocumentClass *klass)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GObjectClass    *gobject_class = G_OBJECT_CLASS (klass);
+	EvDocumentClass *ev_document_class = EV_DOCUMENT_CLASS (klass);
 
 	gobject_class->finalize = tiff_document_finalize;
-}
 
-static gchar *
-tiff_document_get_page_label (EvDocument *document,
-			      EvPage     *page)
-{
-	TiffDocument *tiff_document = TIFF_DOCUMENT (document);
-	static gchar *label;
-	
-	if (TIFFGetField (tiff_document->tiff, TIFFTAG_PAGENAME, &label) &&
-	    g_utf8_validate (label, -1, NULL)) {
-		return g_strdup (label);
-	}
-	
-	return NULL;
-}
-
-static EvDocumentInfo *
-tiff_document_get_info (EvDocument *document)
-{
-	EvDocumentInfo *info;
-
-	info = g_new0 (EvDocumentInfo, 1);
-	info->fields_mask = 0;
-
-	return info;
-}
-
-static void
-tiff_document_document_iface_init (EvDocumentIface *iface)
-{
-	iface->load = tiff_document_load;
-	iface->save = tiff_document_save;
-	iface->get_n_pages = tiff_document_get_n_pages;
-	iface->get_page_size = tiff_document_get_page_size;
-	iface->render = tiff_document_render;
-	iface->get_page_label = tiff_document_get_page_label;
-	iface->get_info = tiff_document_get_info;
+	ev_document_class->load = tiff_document_load;
+	ev_document_class->save = tiff_document_save;
+	ev_document_class->get_n_pages = tiff_document_get_n_pages;
+	ev_document_class->get_page_size = tiff_document_get_page_size;
+	ev_document_class->render = tiff_document_render;
+	ev_document_class->get_page_label = tiff_document_get_page_label;
 }
 
 static GdkPixbuf *
