@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef EV_APPLICATION_H
@@ -30,8 +30,6 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 
-#include "egg-toolbars-model.h"
-
 #include "ev-window.h"
 
 G_BEGIN_DECLS
@@ -50,19 +48,12 @@ typedef struct _EvApplicationClass EvApplicationClass;
 
 GType	          ev_application_get_type	     (void) G_GNUC_CONST;
 EvApplication    *ev_application_get_instance        (void);
-gboolean          ev_application_register_service    (EvApplication   *application);
-void	          ev_application_shutdown	     (EvApplication   *application);
 
+void              ev_application_shutdown            (EvApplication   *application);
 gboolean          ev_application_load_session        (EvApplication   *application);
-gboolean          ev_application_open_window         (EvApplication   *application,
-						      GHashTable      *args,
-						      guint32          timestamp,
-						      GError         **error);
-gboolean          ev_application_open_uri            (EvApplication   *application,
-						      const char      *uri,
-						      GHashTable      *args,
-						      guint            timestamp,
-						      GError         **error);
+void              ev_application_open_window         (EvApplication   *application,
+						      GdkScreen       *screen,
+						      guint32          timestamp);
 void              ev_application_open_uri_at_dest    (EvApplication   *application,
 						      const char      *uri,
 						      GdkScreen       *screen,
@@ -74,23 +65,20 @@ void	          ev_application_open_uri_list       (EvApplication   *application,
 		  			              GSList          *uri_list,
 						      GdkScreen       *screen,
     						      guint32          timestamp);
-GList		 *ev_application_get_windows	     (EvApplication   *application);
+gboolean	  ev_application_has_window	     (EvApplication   *application);
+const gchar *     ev_application_get_uri             (EvApplication   *application);
 GObject		 *ev_application_get_media_keys	     (EvApplication   *application);
 
-EggToolbarsModel *ev_application_get_toolbars_model  (EvApplication   *application);
-void              ev_application_save_toolbars_model (EvApplication   *application);
-void 		  ev_application_set_chooser_uri     (EvApplication   *application, 
+void 		  ev_application_set_filechooser_uri (EvApplication   *application,
+						      GtkFileChooserAction action,
 						      const gchar     *uri);
-const gchar	 *ev_application_get_chooser_uri     (EvApplication   *application);
+const gchar	 *ev_application_get_filechooser_uri (EvApplication   *application,
+						      GtkFileChooserAction action);
 void		  ev_application_screensaver_enable  (EvApplication   *application);
 void		  ev_application_screensaver_disable (EvApplication   *application);
-GtkPrintSettings *ev_application_get_print_settings  (EvApplication   *application);
-void              ev_application_set_print_settings  (EvApplication   *application,
-						      GtkPrintSettings *settings);
-GtkPageSetup     *ev_application_get_page_setup      (EvApplication   *application);
-void              ev_application_set_page_setup      (EvApplication   *application,
-						      GtkPageSetup    *page_setup);
-const gchar      *ev_application_get_dot_dir         (EvApplication   *application);
+const gchar      *ev_application_get_dot_dir         (EvApplication   *application,
+                                                      gboolean         create);
+const gchar      *ev_application_get_data_dir        (EvApplication   *application);
 
 G_END_DECLS
 
